"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var vm_1 = require("vm");
var fs_1 = require("fs");
var path_1 = require("path");
var assert = require("power-assert");
var MousetrapStub = (function () {
    function MousetrapStub() {
        this.handlers = {};
    }
    MousetrapStub.prototype.bind = function (key, handler) {
        this.handlers[key] = handler;
    };
    MousetrapStub.prototype.reset = function () {
        this.handlers = {};
    };
    MousetrapStub.prototype.dispatch = function (key) {
        if (this.handlers[key]) {
            this.handlers[key]();
            return true;
        }
        else {
            return false;
        }
    };
    return MousetrapStub;
}());
var stub = new MousetrapStub();
global.Mousetrap = stub;
var dependant_script = path_1.join(__dirname, '..', '..', '..', '..', 'build', 'src', 'renderer', 'keyboard.js');
vm_1.runInThisContext(fs_1.readFileSync(dependant_script, 'utf8'));
describe('Keyboard.Receiver', function () {
    afterEach(function () {
        stub.reset();
    });
    describe('#constructor()', function () {
        it('registers given shortcuts', function () {
            var shortcuts = {
                j: 'PageDown',
                k: 'PageUp',
                down: 'PageDown',
                up: 'PageUp',
            };
            var r = new Keyboard.Receiver(shortcuts);
            assert(r);
            assert(Object.keys(stub.handlers).length === 4);
            assert('j' in stub.handlers);
            assert('k' in stub.handlers);
            assert('down' in stub.handlers);
            assert('up' in stub.handlers);
        });
        it('enables itself by default', function () {
            var r = new Keyboard.Receiver({});
            assert(r.enabled);
        });
    });
    describe('#on()', function () {
        it('registers a handler when specific action is fired', function () {
            var shortcuts = {
                j: 'Action1',
                k: 'Action2',
            };
            var r = new Keyboard.Receiver(shortcuts);
            var flag1 = false;
            r.on('Action1', function () {
                flag1 = true;
            });
            var flag2 = false;
            r.on('Action2', function () {
                flag2 = true;
            });
            assert(stub.dispatch('j'));
            assert(flag1);
            assert(!flag2);
        });
    });
    describe('#dispatch_shortcut()', function () {
        it('dispatches a handler related specified action', function () {
            var shortcuts = {
                j: 'Action1',
                k: 'Action2',
            };
            var r = new Keyboard.Receiver(shortcuts);
            var flag = false;
            r.on('Action1', function () {
                flag = true;
            });
            r.dispatch_shortcut('Action1');
            assert(flag);
        });
        it('does not fire no handler on unknown action', function () {
            var shortcuts = {
                j: 'Action1',
                k: 'Action2',
            };
            var r = new Keyboard.Receiver(shortcuts);
            var flag = false;
            r.on('Action1', function () {
                flag = true;
            });
            r.dispatch_shortcut('UnknownAction');
            assert(!flag);
        });
    });
    describe('#enabled', function () {
        it('prevent handlers from being called if false', function () {
            var shortcuts = {
                j: 'Action1',
                k: 'Action2',
            };
            var r = new Keyboard.Receiver(shortcuts);
            var flag = false;
            r.enabled = false;
            r.on('Action1', function () {
                flag = true;
            });
            r.dispatch_shortcut('Action1');
            assert(!flag);
            r.enabled = true;
            r.dispatch_shortcut('Action1');
            assert(flag);
        });
    });
});
//# sourceMappingURL=keyboard_test.js.map